/*
* CGI-interface for JVisu socket server
*
* Copyright (C) 2003 Thomas Gleixner <tglx@linutronix.de>
*		
* PURPOSE
* Simple CGI interface to JVisu socket server. Used from JVisu applets,
* if a socket connection cannot be established, e.g. if you access it
* behind a firewall.
*
*/

/*
* LICENCING INFORMATION
*
* JVisu is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License version 2 as published by the
* Free Software Foundation.
*
* JVisu is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along
* with JVisu; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <limits.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <jvisu.h>

char	answer [4096];

/*
*	Try to do this as simple as possible
*/
int main (int argc, char *argv[])
{
	char			*p1, *p2, buf[256]; 
	struct jvisu_regvar	*var = (struct jvisu_regvar *) buf;
	struct jvisu_sndvar	*rec = (struct jvisu_sndvar *) buf;
	struct sockaddr_in 	saddr;
	struct hostent 		*hostinfo;
	struct timeval		timeo;
	int			port, sock = -1, len, cnt = 0;
	fd_set			rfds;
	
	p1 = getenv("QUERY_STRING");
	fprintf (stdout,"Content-type: text/plain\n\n");
	if (p1 == NULL) 
		goto err;
	
	// get portnumber
	p2 = strchr (p1, ';');
	if (p2 == NULL)
		goto err;
	*p2++ = 0x0;
	port = atoi (p1);
	p1 = p2;	
		
	// try to connect to JVisu server
	hostinfo = gethostbyname ("localhost");
	if (hostinfo == NULL)
		goto err;

	saddr.sin_addr = *(struct in_addr *)hostinfo->h_addr;
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons (port);

	// create socket
	sock = socket (AF_INET, SOCK_STREAM, 0);
	if (sock < 0) 
		goto err;
			
	// connect
	if (connect (sock, (struct sockaddr *) &saddr, sizeof (saddr)) < 0) {
		close (sock);
		goto err;
	}
	
	strcpy (answer, "0;");
	cnt = 2;
	FD_ZERO (&rfds);

	// Get them all
	while (*p1 != '\0') {
		if (strchr (p1, ';') == 0)
			break;
		// First token is symbolname	
		p2 = strchr (p1, ',');	
		if (p2 == NULL)
			break;
		*p2++ = 0x0;	
		strcpy (var->symbol, p1);
		// Next token is index
		p1 = strchr (p2, ',');	
		if (p1 == NULL)
			break;
		*p1++ = 0x0;	
		var->idx = cpu_to_je32 (atoi (p2));
		// Last token is symbolname	
		p2 = strchr (p1, ';');	
		if (p2 == NULL)
			break;
		*p2++ = 0x0;	
		var->hdr.type = (u_char) atoi (p1);
		p1 = p2;
		// Register it
		var->rate = cpu_to_je32 (100000);
		var->hdr.cmd = CMD_REG_VAR;
		len = sizeof (struct jvisu_regvar) + strlen (var->symbol);
		var->hdr.len = cpu_to_je16 ((ushort) len);
		if (write (sock, buf, len) != len)
			goto err;
		
		// Read the result
		FD_SET (sock, &rfds);
		timeo.tv_sec = 1;
		timeo.tv_usec = 0;
		if (select(sock + 1, &rfds, NULL, NULL, &timeo) <= 0)
			goto err;
		if (read (sock, buf, sizeof (struct jvisu_cmdhdr)) != sizeof (struct jvisu_cmdhdr))
			goto err;
		len = ((int) je16_to_cpu (rec->hdr.len)) - sizeof (struct jvisu_cmdhdr);
		timeo.tv_sec = 0;
		timeo.tv_usec = 500000;
		if (select(sock + 1, &rfds, NULL, NULL, &timeo) <= 0)
			goto err;
		if (read (sock, &buf[sizeof (struct jvisu_cmdhdr)], len) != len)
			goto err;
		
		switch (rec->hdr.type) {
		case VAR_TYPE_DOUBLE: {
			uint64_t tmp = je64_to_cpu (rec->data.d[0]);
			sprintf (&answer[cnt], "%d,%f;", je32_to_cpu (rec->idx), *((double *) &tmp));
			break;
		}	
		case VAR_TYPE_FLOAT: {
			uint32_t tmp = je32_to_cpu (rec->data.i[0]);
			sprintf (&answer[cnt], "%d,%f;", je32_to_cpu (rec->idx), *((float *) &tmp));
			break;
		}	
		case VAR_TYPE_INT:
			sprintf (&answer[cnt], "%d,%d;", je32_to_cpu (rec->idx), (int) je32_to_cpu (rec->data.i[0]));
			break;
		case VAR_TYPE_SHORT:
			sprintf (&answer[cnt], "%d,%d;", je32_to_cpu (rec->idx), (short) je16_to_cpu (rec->data.s[0]));
			break;
		case VAR_TYPE_BOOL:
		case VAR_TYPE_BYTE:
			sprintf (&answer[cnt], "%d,%d;", je32_to_cpu (rec->idx), rec->data.b[0]);
			break;
		// Do it sometimes :)
		case VAR_TYPE_STRING:
		default:
			continue;
		}
		cnt = strlen (answer);
	}
	goto out;	
err:
	strcpy (answer, "-1;\n");
out:
	if (sock >= 0)
		close (sock);
	fprintf (stdout, "%s\n", answer);
	exit (0);	
}
