/* 
 * jvtree - Lightweight CGI tree widget in C
 *
 * Copyright (C) 2003 by Pengutronix e.K. 
 *
 * Written by Thomas Bangel <t.bangel@pengutronix.de>
 *            Robert Schwebel <r.schwebel@pengutronix.de>
 *
 * This file is licensed under the GNU General Public License. 
 *
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <libgen.h>
#include <cgi-lib.h>
#include <html-lib.h>
#include <string.h>

const char *img_plus  = "plus.png";
const char *img_minus = "minus.png";
const char *img_empty = "leer.png";

struct tree_item {
	char type;
	char *name;
	struct tree_item *first_child;
	struct tree_item *parent;
	struct tree_item *next_sibling;
	struct tree_item *prev_sibling; 
};
struct tree_item *config_tree;

int usage (void)
{
	fprintf(stderr, "Usage: QUERY_STRING=<configfile_url>?<image_path_url>\n");
	return -1;
}


int main (void)
{
	int ret = 0;
	FILE *config_file;
	char *config_filename;
	char *image_path; 
	char *cgi_query;
	struct tree_item *item, *new_item;
	char *line;
	int nbytes;
	ssize_t bytes_read; 
	
	/* print out CGI header & HTML header */
	mime_header("text/html\n\n");
	printf("<html>\n");
	printf("<body>\n");

	/* parse CGI query string */
	cgi_query = getenv ("QUERY_STRING");
	if (!cgi_query) {
		fprintf (stderr, "error: QUERY_STRING not set\n");
		ret = usage();
		goto out;
	}

	item = malloc (sizeof (struct tree_item));
	memset (item, 0, sizeof (struct tree_item));
	item->type = '+';
	item->name = "Root";
	item->parent = NULL;
	item->first_child =
	config_tree = item;
	
	config_filename = strtok (cgi_query, "?");
	config_file = fopen (config_filename,"r");
	if (!config_file) {
		fprintf(stderr, "error: config file not set.\n");
		ret = usage();	
		goto out;
	}

	image_path      = strtok (NULL, "?"); 
	if (!image_path) {
		fprintf (stderr, "error: image path not set.\n");
		ret = usage();
		goto out;
	}

	/* announce result of query parsing */
	printf ("config_filename = %s<br>\n", config_filename);
	printf ("image_path      = %s<br>\n", image_path);

	/* parse config file */
	line = NULL; 
	while ((bytes_read = getline (&line, &nbytes, config_file)) != EOF) {
//		printf ("line: %s", line);
		for (item->level=0; (line[item->level]==' ' || line[item->level]=='+'); item->level++);
		item->type = line[item->level];
		item->name = &line[item->level+1];
		new_item = malloc (sizeof (struct tree_item));
		memset (new_item, 0, sizeof (struct tree_item));
		item->next = new_item;
		item = new_item;
		line = NULL; 
	}
	

	/* iterate over list */
	item = config_tree;
	while (item->next != 0) {
		int i;
		for (i=0; i<item->level; i++) 
			printf("<img src=\"%s/line.png\">", image_path);
		printf("<img src=\"%s/%s\">", image_path, 
			item->type=='+' ? img_plus : img_minus
		);
		printf("%c", item->type);
		printf(" %s", item->name);
		printf("<br>\n");
		item = item->next;
	}
	printf("done\n");
	
out:
	if (config_file) fclose(config_file);

	printf("</body>\n");
	printf("</html>\n");
	
	return ret;
}
