/*
* Header file for JVisu socket server
*
* Copyright (C) 2003 Thomas Gleixner <tglx@linutronix.de>
*
* For licensing information, see the file 'jvisu.c'
*
*/

#ifndef _JVISU_H
#define _JVISU_H

/** Use the kernel one instead of copying it */
#define _LVM_H_INCLUDE
#include <linux/types.h>
#include <sys/types.h>
#include <asm/byteorder.h>
#include <stdint.h>
#include <bits/types.h>

#include "list.h"

#define MAX_CONNECTIONS		10
#define MAX_BUFLEN		4096
#define MAX_DATA_LEN		(MAX_BUFLEN - sizeof (struct jvisu_sndvar))
#define MAX_STREAM_LEN		256
#define MAX_SYMBOL_LEN		256
#define JVISU_PORT		2200
#define JVISU_INTERVAL		20	/* time between variable checks in ms */

#define STAT_READ		1
#define STAT_WRITE		2

#define CMD_ALIVE		0
#define CMD_REG_VAR		1
#define CMD_UNREG_VAR		2
#define CMD_SEND_VAR		3
#define CMD_CONF_DATA		4

#define VAR_STAT_INIT		0x01
#define VAR_STAT_CHANGED	0x02

#define VAR_READWRITE		0x00
#define VAR_READONLY		0x01
#define VAR_WRITEONLY		0x02

#define VAR_TYPE_DOUBLE		1
#define VAR_TYPE_STRING		2
#define VAR_TYPE_BOOL		3
#define VAR_TYPE_BYTE		4
#define VAR_TYPE_SHORT		5
#define VAR_TYPE_INT		6
#define VAR_TYPE_FLOAT		7
#define VAR_TYPE_STREAM		8
#define VAR_TYPE_BYTEARRAY	9

/*
 * Byteorder stuff, stolen from linux/include/fs/jffs2.h
 */
typedef struct {
	uint64_t v64;
} __attribute__((packed))  jint64_t;

typedef struct {
	uint32_t v32;
} __attribute__((packed))  jint32_t;

typedef struct {
	uint16_t v16;
} __attribute__((packed)) jint16_t;

#define cpu_to_je16(x) ((jint16_t){__cpu_to_le16(x)})
#define cpu_to_je32(x) ((jint32_t){__cpu_to_le32(x)})
#define cpu_to_je64(x) ((jint64_t){__cpu_to_le64(x)})
#define je16_to_cpu(x) (__le16_to_cpu(x.v16))
#define je32_to_cpu(x) (__le32_to_cpu(x.v32))
#define je64_to_cpu(x) (__le64_to_cpu(x.v64))

/** jvisu command header */
struct jvisu_cmdhdr {
	u_char		cmd;
	u_char		type;
	jint16_t	len;
};

/** jvisu register variable command */
struct jvisu_regvar {
	struct jvisu_cmdhdr	hdr;
	jint32_t		idx;
	jint32_t		rate;
	u_char			symbol[0];	
};

/** jvisu stream variable */
struct jvisu_varstream {
	int	len;
	char	data[MAX_STREAM_LEN];
};

/** jvisu send variable command */
struct jvisu_sndvar {
	struct jvisu_cmdhdr	hdr;
	jint32_t		idx;
	union {
		u_char		b[0];
		jint16_t	s[0];
		jint32_t	i[0];
		jint64_t	d[0];
	} data;	
};

/** union to represent the internal variable of the application
 *  when writing 
 */
union jvisu_intvar {
	u_char		b[0];
	short		s[0];
	int		i[0];
	float		f[0];
	double		d[0];
	uint16_t	u16[0];
	uint32_t	u32[0];
	uint64_t	u64[0];
};

/** jvisu generic telegram */
struct jvisu_sndgen {
	struct jvisu_cmdhdr	hdr;
	u_char			data[0];
};

/** socket structure */
struct jvisu_sock {
	int		sd;
	int		status;
	unsigned int	bufpoi;
	unsigned int	cmdlen;
	u_char		buf[MAX_BUFLEN];
};

/** jvisu registered variable */
struct jvisu_var {
	struct list_head 	list;
	struct jvisu_sock	*sd;		/* socket descriptor */
	int			type;		/* changed by jvisu  */
	int			inttype;	/* internal type     */
	int			modifytype;
	void 			(*write_done) (int arg);
	int			arg;
	int			idx;
	int			rate;
	int			elapsed;
	int			status;
	int			arraylen;
	void			*data;
	u_char			bitmask;
	u_char			dummy[3];
	union {
		u_char		b;
		uint16_t	s;
		uint32_t	i;
		float		f;
		double		d;
		u_char		*p;
	} cmp;
};

/** jvisu user supplied functions */
struct jvisu_ops {
	int 	(*get_symbol) (struct jvisu_var *var, char *symbol);
	int	(*save_configs) (int typ, char *data);
};

/* Macros for min/max.  */
#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif

/* Stop server */
extern int jvisu_stop_server (int stop);

/* Debug */
extern int jvisu_debuglevel (int level);

/* Run server */
extern int jvisu_socket (struct jvisu_ops *ops, int port);

#endif
