package org.jvisu.comm.var;

import java.nio.*;

/** JVisu variable representing a Double.
 *
 * <p>package:  org.jvisu.comm.var</p>
 * <p>file:    JVVarDouble.java</p>
 * <p>$Id: JVVarDouble.java,v 1.2 2004/04/19 08:20:55 tgl Exp $ </p>
 * <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See file LICENCE in this
 * directory</p>
 *
 * <p>Change Log:</p>
 * <ul>
 * <li> 1.0 Initial inmplimentation </li>
 * </ul>
 * <p>
 * Implementation of abstract class JVVar representing a Double variable for
 * communication with a JVisu communication server
 * </p>
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVVarDouble extends JVVar {
    
    double value;
    
    /** Creates a new instance of JVVarDouble 
     */
    public JVVarDouble() {
    }
    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean SetVarValue (String str) {
        try {
            return setVarValue(Double.parseDouble(str));
        } catch ( NumberFormatException e ) {
            return false;
        }  
    }
    
    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
    */
    public boolean SetVarValue(ByteBuffer buf) {
        return setVarValue(buf.getDouble());
    }

    /** Set the variable value from an object
     *  @param obj  Object containing the variable value
     *  @return     true, if successful
    */
    public boolean SetVarValue(Object obj) {
        try {
            double val = ((Double) obj).doubleValue();
            return setVarValue (val);
        } catch (ClassCastException e) {
            return false;
        }
    }

    /** Set the variable value from a double
     * @return true, if successful
     * @param newval the new variable value
     */
    public boolean setVarValue (double newval) {
        double oldval = value;
        value = newval;
        NotifyListeners((value != oldval));
        return true;
    }
    
    /** Get the variable value as double
     *  @return     variable value
     */
    public double getVarValue () {
        return value;
    }
    
    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public ByteBuffer GetVarValue(ByteBuffer buf) {
        buf.putDouble(value);
        return buf;
    }
    
    /** Get the string representation of the variable
     *  @return     variable value as string
     */
    public String toString () {
        return Double.toString(value);
    }
}
