/*
 * SplashScreen.java - Splash screen
 * Copyright (C) 1998, 1999, 2000, 2001 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jvisu.panel.configurator;

import java.awt.*;

/**
 * The splash screen displayed on startup.<p>
 *
 * This file only uses AWT APIs so that it can be displayed as soon as possible
 * after start of the program.
 */
public class SplashScreen extends Canvas {
    
    /** Create a new instance
     */
    public SplashScreen() {
        
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        
        Font font = new Font("Dialog",Font.PLAIN,20);
        setFont(font);
        fm = getFontMetrics(font);
        
        image = getToolkit().getImage(
        getClass().getResource("/org/jvisu/panel/configurator/icons/splash.png"));
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image,0);
        
        try {
            tracker.waitForAll();
        } catch(Exception e) { }
        
        win = new Window(new Frame());
        
        Dimension screen = getToolkit().getScreenSize();
        Dimension size = new Dimension(image.getWidth(this) + 2,
        image.getHeight(this) + 2);
        win.setSize(size);
        
        win.setLayout(new BorderLayout());
        win.add(BorderLayout.CENTER,this);
        
        win.setLocation((screen.width - size.width) / 2,
        (screen.height - size.height) / 2);
        win.validate();
        win.show();
    }
    
    public void dispose() {
        win.dispose();
    }
    
    public synchronized void advance() {
        progress++;
        repaint();
        
        // wait for it to be painted to ensure progress is updated
        // continuously
        try {
            wait();
        }
        catch(InterruptedException ie) {
        }
    }
    
    public void update(Graphics g) {
        paint(g);
    }
    
    public synchronized void paint(Graphics g) {
        Dimension size = getSize();
        
        if(offscreenImg == null) {
            offscreenImg = createImage(size.width,size.height);
            offscreenGfx = offscreenImg.getGraphics();
            offscreenGfx.setFont(getFont());
        }
        
        offscreenGfx.setColor(Color.GRAY);
        offscreenGfx.drawRect(0,0,size.width - 1,size.height - 1);
        
        offscreenGfx.drawImage(image,1,1,this);
        
        offscreenGfx.setColor(Color.BLUE);
        offscreenGfx.fillRect(10,image.getHeight(this) - PROGRESS_HEIGHT - 1,
        ((win.getWidth() - 20) * progress) / 40, PROGRESS_HEIGHT);
        
        String str = JVPanelConfigurator.getVersion();
        
        offscreenGfx.drawString(str,
        (getWidth() - fm.stringWidth(str)) / 2,
        image.getHeight(this)+1 - fm.getDescent() - 10);
        
        g.drawImage(offscreenImg,0,0,this);
        
        notify();
    }
    
    // private members
    private FontMetrics fm;
    private Window win;
    private Image image;
    private Image offscreenImg;
    private Graphics offscreenGfx;
    private int progress;
    private static final int PROGRESS_HEIGHT = 5;
    
    private static SplashScreen splash;
    
    /**
     * Ensures that the splash screen is not visible. This should be
     * called before displaying any dialog boxes or windows at
     * startup.
     */
    public static void hideSplashScreen() {
        if(splash != null) {
            splash.dispose();
            splash = null;
        }
    }
    
    /** Create and show a new SplashScreen
     */
    public static void showSplashScreen() {
        splash = new SplashScreen();
    }
    
    /** Increment the progress and update screen
     */
    public static void advanceSplashProgress() {
        if(splash != null)
            splash.advance();
    }
}

